#include "CMessage.h"
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
//  CMessage.cpp   Encapsulating the struct of message in a class//
//                                                               //
//  ver 1.0                                                      //
//                                                               //
//  Language:        Visual C, ver 6.0                           //
//  Platform:        Micron Dual Pentium Pro 200, Win NT 4.0     //
//  Application:     CSE691 Project #2                           //
//  Author:       Jim Fawcett, CST 2-187, Syracuse Univ					 //
//                (315) 443-3948, fawcett@syr.edu                // 
//  Modified by:     Jianxiang Jin            CSE691, Fall '99   //
//                   Syracuse University       (315) 443-1053    //
//                   jjin@syr.edu                                //
//                                                               //
///////////////////////////////////////////////////////////////////

using namespace std;

//return id
clientIDtype& CMessage::getClientID(){
	return clientID;
}

//return msg type
msgType&	CMessage::getMsgType(){
	return mt;
}

//return length
int&	CMessage::getLength(){
	return length;
}

//set length
void	CMessage::setLength(int l){
	length =l;
}

//get block[i]
char&	CMessage::getBlockElement(int i){
	return block[i];
}

//set block[i]
void  CMessage::setBlockElement(int i,char c){
	block[i]=c;
}

//get block
char* CMessage::getBlockArray(){
	return &block[0];
	//return block;
}

//constructor
CMessage::CMessage() : clientID(-1), mt(generic), length(0) {
  int i;
  for(i=0; i<BlockSize; i++)
    block[i] = '@';
  sentinal = '@';
}

/*
CMessage::CMessage(clientIDtype ID, msgType mst, int len, char blck[]) {
  clientID = ID;
  mt = mst;
  length = len;
  int i;
  for(i=0; i<length; i++)
    block[i] = blck[i];
  for(i=length; i<BlockSize; i++)
    block[i] = '@';
  sentinal = '@';
  if(!invariant())
	    errorOut("\n  ---- message fails invariant test in constructor---\n", true );

}*/
bool CMessage::operator==(const CMessage &m) const {
  if(clientID != m.clientID) return false;
  if(      mt != m.mt      ) return false;
  if(  length != m.length  ) return false;
  int i;
  for(i=0; i<length; i++)
    if(block[i] != m.block[i]) return false;
  return true;
  if(sentinal != m.sentinal ) return false;
}

bool CMessage::operator!=(const CMessage &m) const {
  if(*this==m)  return false;
  return true;
}
CMessage& CMessage::operator= (const CMessage &m) {
  if(this == &m) return *this;
  clientID = m.clientID;
  mt = m.mt;
  length = m.length;
  int i;
  for(i=0; i<length; i++)
    block[i] = m.block[i];
  return *this;
}

CMessage::CMessage(const CMessage &m) {
  clientID = m.clientID;
  mt = m.mt;
  length = m.length;
  int i;
  for(i=0; i<length; i++)
    block[i] = m.block[i];
  sentinal = '@';
}

//test invariant
bool CMessage::invariant() const {
  if(length<0 || BlockSize<length) return false;
  if(sentinal != '@') return false;
  return true;
}


const char* CMessage::msgTypeToString(msgType mt) {
  const int numTypes = 9;
  const char msgType[numTypes] = {
    disconnect,     // ending conversation
    generic,        // for testing
    send_file,      // request for file, name in message
    listening,      // listening for messages - recvr must send msg
    done,           // no more requests, will wait for replies
    file_data,      // a block of file data is enclosed
    file_error,     // file does not exist
    end_file,       // no more data in file
    shutDownServer  // any client can shut down server
  };
  static const char* msgString[numTypes] = {"disconnect", "generic", "send_file", "listening", "done",
    "file_data", "file_error", "end_file", "shutDownServer" };
  int i;
  for(i=0; i<numTypes; i++)
    if(mt == msgType[i])
      return msgString[i];
  return "unknown message type";
}

//----< write string into message block >----------------------
//
//  - writes the string passsed by the user into the objects message block
//  - checks invariant for error
//===============================================================
void CMessage::stringToMsgBlock(CMessage &m, const char *s) {
  m.length = strlen(s);
  m.length = Min(BlockSize-1,m.length);
  int i;
  for(i=0; i<m.length; i++)
    m.block[i] = s[i];
  m.block[m.length] = '\0';
  if(!m.invariant())
    errorOut(
      "\n  ---- message fails invariant test in stringToMsgBlock ---\n",
      true
    );
}

//----< write string into message block >----------------------
//
//  - writes the string passsed by the user into the objects message block
//  - checks invariant for error
//===============================================================

string CMessage::msgBlockToString(CMessage &m) {

  string temp;
  int i;
  for(i=0; i<m.length; i++)
    temp += m.block[i];

  if(!m.invariant())
    errorOut(
      "\n  ---- message fails invariant test in msgBlockToString ---\n",
      true
    );

  return temp;
}

CMessage CMessage::buildMsg(clientIDtype ID, msgType mt, int length, char block[]) {

  CMessage m;
  m.clientID = ID;
  m.mt = mt;
  m.length = length;
  int i;
  for(i=0; i<length; i++)
    m.block[i] = block[i];
  if(!m.invariant())
    errorOut(
      "\n  ---- message fails invariant test in buildMsg ---\n",
      true
    );
  return m;
}
/*

void CMessage::buildMsg(clientIDtype ID, msgType mt, int length, char block[]) {
  
  clientID = ID;
  mt = mt;
  length = length;
  int i;
  for(i=0; i<length; i++)
    block[i] = block[i];
  if(!invariant())
    errorOut(
      "\n  ---- message fails invariant test in buildMsg ---\n",
      true
    );  
}
*/
/*
	display the message
*/
void CMessage::showMsg(const char *text) {

  ostrstream format;
  if(!format.good())
    errorOut(
      "\n  --- strstream ctor failure in showMsg(...) ---\n",
      true
    );
  //if(dt == border)
    format << "\n =============================================";
  format << "\n  "                    << text
         << "\n  client ID        = " << clientID
         << "\n  message type     = " << msgTypeToString(mt)
         << "\n  message length   = " << length
         << "\n  message contents = ";
  int i;
  for(i=0; i<length; i++)
    format << block[i];
  format << "\n";
  //if(dt == border)
    format << " =============================================\n";
  format << ends;

  safeOut(format.str());

  if(!invariant())
    errorOut(
      "\n  ---- message fails invariant test in showMsg ---\n",
      true
    );
}

//test stub
#ifdef TEST_CMESSAGE

void main()
{
	// cerating  amessage
	CMessage m(0,send_file,5,"hello");
	//showing the message
	m.display("First Message ");
	// cerating  another message
	msg m2(0,send_file,5,"how r u");
	//showing the other message
	m2.display("Second message ");
	cout << "Comparing messages \n";
	if(m ==m2)
		cout << "They are equal \n";
	else
		cout << "They are different \n";
}

#endif