#ifndef CMESSAGE_H
#define CMESSAGE_H
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
//  CMessage.cpp   Encapsulating the struct of message in a class//
//                                                               //
//  ver 1.0                                                      //
//                                                               //
//  Language:        Visual C, ver 6.0                           //
//  Platform:        Micron Dual Pentium Pro 200, Win NT 4.0     //
//  Application:     CSE691 Project #2                           //
//  Author:       Jim Fawcett, CST 2-187, Syracuse Univ					 //
//                (315) 443-3948, fawcett@syr.edu                // 
//  Modified by:     Jianxiang Jin            CSE691, Fall '99   //
//                   Syracuse University       (315) 443-1053    //
//                   jjin@syr.edu                                //
//                                                               //
///////////////////////////////////////////////////////////////////
/*

    Module Operations
    =================
		Define the basic operations for socket and CMessage handling.

    Public Interface
    ================
		clientIDtype& getClientID();
		--------return the clientID;

		msgType&	getMsgType();
		--------return the message type

		int&	getLength();
		--------return length

		void	setLength(int l);
		--------set length

		char&	getBlockElement(int i);
		--------get block[i];
		void  setBlockElement(int i,char c);
		--------set block[i]
		char* getBlockArray();			
		--------return block;

		const char* msgTypeToString(msgType mt);
		--------covert msg type to string

*/

//
///////////////////////////////////////////////////////////////
//  build process                                            //
///////////////////////////////////////////////////////////////
//  required files:                                          //
//    Win32utils.h, Win32utils.cpp													 //
//    CMessage.h CMessage.cpp																 //
//		mtqueue.h  mtqueue.cpp																 //		
//		CSocket.h   CSocket.cpp																 //
//                                                           //
//  compiler command line invocation:                        //
//    cl -GX -DTEST_CSocket   CSocket.cpp                    // 
//		CMessage.cpp mtqueue.cpp Win32utils.cpp								 //
///////////////////////////////////////////////////////////////
/*
    Maintenance History
    ===================
    ver 1.0 

*/

#include "Win32utils.h"
#include <strstream>
#include <iostream>
#include <string>
using namespace std;

typedef long int clientIDtype;

const int BlockSize   = 256;  // length of message data area
enum msgType { 
  disconnect,       // ending conversation
  generic,          // for testing
  send_file,        // request for file, name in message
  listening,        // listening for msgs, recvr must send
  done,             // no more requests, will wait for replies
  file_data,        // a block of file data is enclosed
  file_error,       // file does not exist
  end_file,         // no more data in file
  shutDownServer    // any client can shut down server
};


class CMessage{
private:
//public:
	clientIDtype clientID;     // client identifier
	msgType mt;                // message type
	int  length;               // actual size of message contents
	char sentinal;             // mark end of message for invariant();
	char block[BlockSize];     // storage area for message contents
public:
	CMessage();	
	bool operator==(const CMessage &m) const;
	bool operator!=(const CMessage &m) const;
	CMessage& operator= (const CMessage &m);
	CMessage(const CMessage &m);
	bool invariant() const;

	//function to access data
	clientIDtype& getClientID();
	msgType&	getMsgType();
	int&	getLength();
	void	setLength(int l);
	char&	getBlockElement(int i);
	void  setBlockElement(int i,char c);
	char* getBlockArray();

	
	const char* msgTypeToString(msgType mt);
	void stringToMsgBlock(CMessage &message, const char *s);
	std::string msgBlockToString(CMessage &message);
	void showMsg(const char *text);
	CMessage	buildMsg(clientIDtype ID, msgType mt, int length, char block[]);

};

#endif