#include <iostream>
#include <winsock.h>
#include <process.h>
#include "Win32Utils.h"
#include "CServer.h"
using namespace std;
///////////////////////////////////////////////////////////////////
//	CServer.cpp                                                  //
//                                                               //
//  ver 1.0                                                      //
//                                                               //
//  Language:        Visual C, ver 6.0                           //
//  Platform:        Micron Dual Pentium Pro 200, Win NT 4.0     //
//  Application:     CSE691 Project #2                           //
//  Author:         Jianxiang Jin            CSE691, Fall '99    //
//                   Syracuse University       (315) 443-1053    //
//                   jjin@syr.edu                                //
//                                                               //
///////////////////////////////////////////////////////////////////


//----< make socket connection >-------------------------------
//
//  port - specify a service by using known port number
//

void CServer::startServer(int port) {

// define socket parameters for server's listening socket
//   s-addr = INADDR_ANY means the socket will accept any connection

  struct sockaddr_in Address;
  Address.sin_family = AF_INET;
  Address.sin_addr.s_addr = INADDR_ANY;
  Address.sin_port = htons(port);

// bind to listening socket
  
  if(bind(sock, (sockaddr*)&Address, sizeof(Address)) == SOCKET_ERROR) {
    errorOut(" can't bind server to accept any sender\n");
  }
  if(listen(sock,3) == SOCKET_ERROR) {
    errorOut("  can't listen for incoming requests\n");
  }

//

// start server loop to look for connections and serve them

  bool quit = false;   // false until client sends shutDownServer message
  do {
    int size = sizeof(Address);
		CServer newClient;
    //SOCKET clientSock = accept(sock, (sockaddr*)&Address, &size);
		newClient.setSocket(accept(sock, (sockaddr*)&Address, &size));
    quit = newClient.isQuit();
    if(!quit) {
      HANDLE thHand = 
        (HANDLE)_beginthread(socketServer,0,(void*)&newClient);
      if(thHand == INVALID_HANDLE_VALUE) {
        MessageBox(
          NULL,"could not start thread for incoming client",
          "debugging",MB_OK
        );
        break;
      }
      ++numClients;
    }
    // - when socketServer terminates _endthread is called by Win32
    //   which in turn calls closeHandle(thHand), so this code does
    //   not need to do that
    ///////////////////////////////////////////////////////////////////
    // - server's client-handling threads MUST decrement numClients  //
    //   when they terminate or server won't shut down               //
    ///////////////////////////////////////////////////////////////////

  } while(!quit);

// Handling an array of thread handles which have to be reused was 
// somewhat messy.  So I opted not manage handles which implies that 
// I can't wait on them to become signaled.  So I use the ugly polling 
// process, below.

  while(numClients >0)
    Sleep(50);

// shut server down
  closesocket(sock);
  cleanUpSockets();
}

//----< process file messages >--------------------------------
//
//  - gets file name from rMsg, passed by serverWorker caller
//  - opens file if it can, otherwise sends a file_error message
//  - reads file block-by-block directly into CMessage blocks
//  - posts the blocks on an output queue
//
void CServer::processFileMessages(CMessage &rMsg,mtqueue<CMessage> &out) {

  CMessage sMsg;
  DWORD bytesRead;
  char block[BlockSize];
  string fileName = ".\\sData\\";
  //rMsg.block[rMsg.length] = '\0';
	rMsg.setBlockElement(rMsg.getLength(),'\0');
  fileName += rMsg.getBlockArray();

	//safeOut("-----------------begin file\n");
  HANDLE hFile = ::CreateFile(
    fileName.c_str(),GENERIC_READ,0,NULL,OPEN_EXISTING,
    FILE_ATTRIBUTE_NORMAL,NULL
  );
  if(hFile == INVALID_HANDLE_VALUE) {
	//safeOut("-----------------open error\n");
    safeOut("\n--- can't open file ---\n");
    sMsg=sMsg.buildMsg(0,file_error,0,block);
    out.enQ(sMsg);
  }
  else {
	//safeOut("----------1-------\n");
    out.enQ(rMsg);
	//safeOut("----------2-------\n");
    sMsg=sMsg.buildMsg(0,file_data,BlockSize,block);
    do {
      if(::ReadFile(hFile,sMsg.getBlockArray(),BlockSize,&bytesRead,NULL)) {
        if(bytesRead == 0)
          break;
        sMsg.setLength(bytesRead);
        out.enQ(sMsg);
      }
    } while(bytesRead > 0);
    sMsg=sMsg.buildMsg(0,end_file,0,block);
    out.enQ(sMsg);
  }
  CloseHandle(hFile);
	//safeOut("-----------------end file\n");
}

/*
	threads.................

*/



//----< sender thread function >-------------------------------
//
//  - sender's job is to send all messages from out queue
//    back to the client
//  - will keep emptying queue into socket when there is
//    anything to send, otherwise sleeps
//  - only stops when it processes a clients disconnect CMessage
//
void _cdecl sender(void *CQout) {

  comQ *pComQout = (comQ*)CQout;
  bool quit = false;
  //SOCKET sock = (SOCKET)pComQout->sock;
	CServer newClient = (CServer)pComQout->newClient;
  CMessage sMsg;
  int numBytes = 0;

  while(1) {

    while(pComQout->pQ->length() == 0)
      Sleep(50);

    pComQout->pQ->deQ(sMsg);

    if(sMsg.getMsgType() == disconnect) {
      quit = true;
    }
    numBytes += newClient.sendMsg(sMsg);
    if(pComQout->pQ->length() == 0 && quit) {
      break;
    }
  }
  trace("  sent to client a total of ",numBytes," bytes\n");
  trace(
    "  there are ",pComQout->pQ->length(),
    " enqueued out messages"
  );
  safeOut("\n");
}

//----< worker thread >----------------------------------------
//
//  - deQueues from in, builds output CMessage and enQueues on out
//  - here is were all the real server work is done, via
//    the processFileMessages function
//  - only stops when it processes a client's disconnect CMessage, then
//    waits for sender thread to finish
//
void _cdecl serverWorker(void *CQin) {

  comQ *pComQin = (comQ*)CQin;
	//type cast
	CServer newClient =(CServer)pComQin->newClient;
	/*
	CServer* _newClient = (CServer *)&__newClient;
	CServer newClient = *_newClient;
	*/
  mtqueue<CMessage> out;

  comQ comQout;
	comQout.newClient = pComQin->newClient;
  comQout.pQ   = &out;

  HANDLE thSender = (HANDLE)_beginthread(sender,0,(void*)&comQout);
  bool quit = false;
  int fileCount = 0;

  while(pComQin->pQ->length() > 0 || !quit) {

    while(pComQin->pQ->length() == 0)
      Sleep(50);

    CMessage rMsg;
    pComQin->pQ->deQ(rMsg);

    if(rMsg.getMsgType() == send_file && (rMsg.getLength() > 0)) {
      newClient.processFileMessages(rMsg,out);
      ++fileCount;
    }
    else
      out.enQ(rMsg);
    if(rMsg.getMsgType() == disconnect)
      quit = true;
  }
  trace("  processed ",fileCount," files");
  WaitForSingleObject(thSender,INFINITE);
}

//

//--< receiver thread >----------------------------------------
//
//  - messures server performance
//  - collects messages from socket and enqueues them for worker
//  - main purpose is to drain socket so client won't block
//    trying to send messages
//  - only stops when it processes a client's disconnect CMessage, then
//    waits for serverWorker thread to finish
//
void _cdecl socketServer(void *_newClient) {

  //SOCKET sock = (SOCKET)clientSocket;
	CServer newClient = *((CServer *)_newClient);
  mtqueue<CMessage> in;
  CMessage rMsg;

  comQ comQin;
  //comQin.sock = sock;
	comQin.newClient = newClient;
  comQin.pQ   = &in;

  HANDLE thWorker = (HANDLE)_beginthread(serverWorker,0, (void*)&comQin);
  
  //clock_t elapsedTime;

  while(1) {	
    //timer time;
    //time.start();
    int numBytes = newClient.recvMsg(rMsg);
    in.enQ(rMsg);

    if(rMsg.getMsgType() == disconnect) {
      //time.stop();
      //elapsedTime = time.elapsed();
      break;
    }
  }
/*
  trace(
    "\n    Elapsed Time to process all messages = ",
    elapsedTime,
    " milliseconds"
  );

	*/
  WaitForSingleObject(thWorker,INFINITE);
  --numClients;
  newClient.closeSocket();
  trace("  there are ",in.length()," enqueued input messages");
}



//----< entry point >------------------------------------------

void main(int argc, char *argv[]) {
	cout << "\n ----------------------------------------\n";	
	cout << "\n             Welcome ! \n";	
	cout << "\n          Server is started\n"	;		
	cout << "\n          Listening port 1024....\n"	;		
	cout << "\n         Jianxiang Jin    jjin@syr.edu  		\n";	
	cout << "\n ----------------------------------------\n";  			
	CServer s;
  initializeSockets();
  initializeUtils();
  s.createSocket();
	//begin listening.....................
  s.startServer(1024);  
  cleanUpUtils();
  cleanUpSockets();
	//close server socket
  s.closeSocket();
  cout << endl;
}
