#ifndef CSOCKET_H
#define CSOCKET_H
///////////////////////////////////////////////////////////////////
//	CSocket.h                                                    //
//                                                               //
//  ver 1.0                                                      //
//                                                               //
//  Language:        Visual C, ver 6.0                           //
//  Platform:        Micron Dual Pentium Pro 200, Win NT 4.0     //
//  Application:     CSE691 Project #2                           //
//  Author:       Jim Fawcett, CST 2-187, Syracuse Univ					 //
//                (315) 443-3948, fawcett@syr.edu                // 
//  Modified by:     Jianxiang Jin            CSE691, Fall '99   //
//                   Syracuse University       (315) 443-1053    //
//                   jjin@syr.edu                                //
//                                                               //
///////////////////////////////////////////////////////////////////
/*

    Module Operations
    =================
		Define the basic operations for socket and CMessage handling.

    Public Interface
    ================
		void createSocket();
		-------createSocket;

		void setSocket(SOCKET _s);
		-------used by server accept client

		void closeSocket();
		-------close the socket

		bool	isQuit();	
		-------test whether the connection is still there

		int  sendMsg(const CMessage &m);
		-------send message to socket

 		int  recvMsg(CMessage &mess);	
		-------receive message from socket

*/

//
///////////////////////////////////////////////////////////////
//  build process                                            //
///////////////////////////////////////////////////////////////
//  required files:                                          //
//    Win32utils.h, Win32utils.cpp													 //
//    CMessage.h CMessage.cpp																 //
//		mtqueue.h  mtqueue.cpp																 //		
//		CSocket.h   CSocket.cpp																 //
//                                                           //
//  compiler command line invocation:                        //
//    cl -GX -DTEST_CSocket   CSocket.cpp                    // 
//		CMessage.cpp mtqueue.cpp Win32utils.cpp								 //
///////////////////////////////////////////////////////////////
/*
    Maintenance History
    ===================
    ver 1.0 

*/


#include <iostream>
#include <strstream>
#include <winsock.h>
//#include <process.h>
#include "Win32Utils.h"
#include "CMessage.h"
#include "mtqueue.h"



// length of message data area
#define BlockSize  256
// number of bytes in largest data type in message
#define BucketSize  8
class CSocket{
protected:
	bool quit;
	SOCKET sock;
	// will attempt this many re-sends
	int sendRetries;  
	// will attempt this many re-recvs
	int recvRetries;  
	//get client id
	char* getMsgClientID(char buffer[], clientIDtype &ID);
	//get msg type
	char* getMsgType(char buffer[], msgType &mt);
	//get msg length
	char* getMsgLength(char buffer[], int &length);
	//send them all out
	int		sendAll(const char *block, int blockLen);
	//receive
	int		recvAll(char *block, int blockLen);
	int		pack(char buffer[], int bufferLen, CMessage m);
	int		unpack(char buffer[], int bufferLen, CMessage &m);
public:
	CSocket();
	void createSocket();
	void setSocket(SOCKET _s);
	void closeSocket();
	bool	isQuit();	
	int  sendMsg(const CMessage &m);
	int  recvMsg(CMessage &mess);		
};
#endif