#include "CSocket.h"
///////////////////////////////////////////////////////////////////
//	CSocket.cpp                                                  //
//                                                               //
//  ver 1.0                                                      //
//                                                               //
//  Language:        Visual C, ver 6.0                           //
//  Platform:        Micron Dual Pentium Pro 200, Win NT 4.0     //
//  Application:     CSE691 Project #2                           //
//  Author:       Jim Fawcett, CST 2-187, Syracuse Univ					 //
//                (315) 443-3948, fawcett@syr.edu                // 
//  Modified by:     Jianxiang Jin            CSE691, Fall '99   //
//                   Syracuse University       (315) 443-1053    //
//                   jjin@syr.edu                                //
//                                                               //
///////////////////////////////////////////////////////////////////

using namespace std;

CSocket::CSocket(){
	quit = false;
	//numClients = 0;
	sendRetries = 100;  
	recvRetries =  10;  
}

//----< load type value from byte array >----------------------
//
//  - pBlob is a pointer into a blob array where t's value is stored
//  - t is the location where value will be loaded from buffer
//  - returns pointer to next byte in blob after t's value
//  - Note: BucketSize is defined in header so a user of this class
//          can make it bigger if needed.
//
template <class T>
char* blobToType(char *pBlob, T &t) {

  union { char bucket[BucketSize]; T newT; } grundge;
  int i, len = sizeof(T);
  for(i=0; i<len; i++) {
    if(i >= BucketSize)
      errorOut("overran bucket size in blobToType");
    grundge.bucket[i] = pBlob[i];
  }
  t = grundge.newT;
  return (pBlob+len);
}

//----< load byte array from type value >----------------------
//
//  - pBlob is a pointer into a blob array where values will be stored
//  - t is the value that will be loaded into buffer
//  - returns pointer to next byte in blob after t's value
//  - Note: BucketSize is defined in header so a user of this class
//          can make it bigger if needed.
//
template <class T>
char* typeToBlob(char *pBlob, T &t) {

  union { char bucket[BucketSize]; T newT; } grundge;
  grundge.newT = t;
  int i, len = sizeof(T);
  for(i=0; i<len; i++) {
    if(i >= BucketSize)
      errorOut("overran bucket size in blobToType");
    pBlob[i] = grundge.bucket[i];
  }
  return (pBlob+len);
}

//

//----< get client ID from packed buffer >---------------------

char* CSocket::getMsgClientID(char buffer[], clientIDtype &ID) {

  char *pBuffer = buffer;
  return blobToType(pBuffer,ID);
}

//----< get message type from packed buffer >------------------

char* CSocket::getMsgType(char buffer[], msgType &mt) {

  clientIDtype ID;
  char *pBuffer = getMsgClientID(buffer,ID);
  return blobToType(pBuffer,mt);
}

//----< get message length from packed buffer >----------------

char* CSocket::getMsgLength(char buffer[], int &length) {

  clientIDtype ID;
  msgType mt;
  char *pBuffer = getMsgClientID(buffer,ID);
  pBuffer = getMsgType(buffer,mt);
  return blobToType(pBuffer,length);
}

//

//----< pack message contents into front of buffer >-----------
//
//  pack structured message into an array of bytes
//  to send over socket connection
//
int CSocket::pack(char buffer[], int bufferLen, CMessage m) {

// pack clientID

  char *pBuffer = buffer;
  pBuffer = typeToBlob(pBuffer,m.getClientID());

// pack message type

  pBuffer = typeToBlob(pBuffer,m.getMsgType());

// pack block length value

  if(m.getLength() < 0)
    m.setLength(0);
  pBuffer = typeToBlob(pBuffer,m.getLength());

// pack block contents

  int i;
  for(i=0; i<m.getLength(); i++)
    *(pBuffer++) = m.getBlockElement(i);

  if(!m.invariant())
    errorOut(
      "\n  ---- message fails invariant test in pack ---\n",
      true
    );

// return size of buffer contents

  return (pBuffer-buffer);
}

//

//----< unpack message contents from front of buffer >---------
//
//  extract structured message from an array of bytes
//  sent over socket connection
//
int CSocket::unpack(char buffer[], int bufferLen, CMessage &m) {

// unpack message parameters

  char *pBuffer = getMsgClientID(buffer,m.getClientID());
  pBuffer = getMsgType(buffer,m.getMsgType());
  pBuffer = getMsgLength(buffer,m.getLength());

// unpack message block contents
  
  int i;
  for(i=0; i<m.getLength(); i++)
    m.setBlockElement(i, *(pBuffer++));

  if(!m.invariant())
    errorOut(
      "\n  ---- message fails invariant test in unpack ---\n",
      true
    );

  return (pBuffer-buffer);
}

//----< create socket >----------------------------------------

void CSocket::createSocket() {

  sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	
  if(sock == INVALID_SOCKET)
    errorOut("can't create socket");
  //return sock;
}



void CSocket::setSocket(SOCKET _s){
	sock = _s;
}

//----< keep sending until socket has eaten all bytes >--------

int CSocket::sendAll(const char *block, int blockLen) {
  int bytesSent;
  int bytesLeft = blockLen;
  int blockIndx = 0;
  int count = 0;
  while(bytesLeft > 0) {
    bytesSent = send(sock,&block[blockIndx],bytesLeft,0);
    if(bytesSent == SOCKET_ERROR) {
      Sleep(50);
      ++count;
    }
    if(count==sendRetries) {
      trace("\n  --- sendMsg failed after ",sendRetries," attempts ---\n");
      return (blockLen-bytesLeft);
    }
    bytesLeft -= bytesSent;
    blockIndx += bytesSent;

    #ifdef DEBUG
    if(bytesLeft > 0)
      trace("\n  --- ",bytesLeft," bytes left in sendAll ---\n");
    #endif

  }
  return blockLen;
}

//----< send contents of block over socket connection >--------

int CSocket::sendMsg(const CMessage &m) {

  int numBytes, count=0;
  const int BufSize = sizeof(CMessage) + BlockSize;
  char buffer[BufSize];
  int len = pack(buffer,sizeof(CMessage),m);
  numBytes = sendAll(buffer,len);

  if(!m.invariant())
    errorOut(
      "\n  ---- message fails invariant test in sendMsg ---\n",
      true
    );

  return numBytes;
}

//
//----< keep receiving until socket has released all bytes >---

int CSocket::recvAll(char *block, int blockLen) {
  int bytesRecvd, bytesLeft = blockLen;
  int blockIndx = 0, count = 0;
  while(bytesLeft > 0) {
    bytesRecvd = recv(sock,&block[blockIndx],bytesLeft,0);
    if(bytesRecvd == SOCKET_ERROR) {
      Sleep(50);
      ++count;
    }
    if(count==recvRetries) {
      trace("\n  --- recvMsg failed after ",recvRetries," attempts ---\n");
      closeSocket();
      exit(1);
    }
    bytesLeft -= bytesRecvd;
    blockIndx += bytesRecvd;

    #ifdef DEBUG
    if(bytesLeft > 0)
      trace("\n  --- ",bytesLeft," bytes left in recvAll ---\n");
    #endif
  }
  return blockLen;
}

//----< collect contents of block from socket buffer >---------

int CSocket::recvMsg(CMessage &m) {
  const int BufSize = sizeof(CMessage) + BlockSize;
  char buffer[BufSize];

// collect message clientID, type, and length

  int len = sizeof(m.getClientID()) + sizeof(m.getMsgType()) + sizeof(m.getLength());
  int numBytes, count=0;
  numBytes = recvAll(buffer,len);
  char *pBuffer = getMsgClientID(buffer,m.getClientID());
  pBuffer = getMsgType(buffer,m.getMsgType());
  pBuffer = getMsgLength(buffer,m.getLength());

// using length, collect message contents, then unpack message

  if(m.getLength() > 0)
    numBytes += recvAll(pBuffer,m.getLength());
  unpack(buffer,sizeof(CMessage),m);
  if(!m.invariant())
    errorOut(
      "\n  ---- message fails invariant test in recvMsg ---\n",
      true
    );
  return numBytes;
}
//

//----< is a shut down message? >------------------------------
//
//  peek in listener's socket buffer to see if this is a
//  shut down message
//
//bool CSocket::isQuit(SOCKET &sock) {
bool CSocket::isQuit() {
  char buffer[sizeof(CMessage)];
  CMessage rMsg;

  int numBytes = recv(sock,buffer,sizeof(buffer),MSG_PEEK);
  unpack(buffer,sizeof(CMessage),rMsg);
  rMsg.showMsg("listener peeking at new client's  message");
  if(rMsg.getMsgType() == shutDownServer) {
    #ifdef VERBOSE
      tracer("  recv ",numBytes," bytes from client ",rMsg.clientID);
      rMsg.showMsg("received message:");
    #endif
    return true;
  }
  return false;
}

//----< close a specific socket >------------------------------

void CSocket::closeSocket() { closesocket(sock); }



//test stub
#ifdef TEST_CSOCKET

void main() {
  initializeSockets();
  initializeUtils();

	CSocket c;
	c.createSocket();
	c.connectToServer("127.0.0.1");

  cleanUpUtils();
  cleanUpSockets();
	c.closeSocket();

}

#endif


